@@ -1204,8 +1204,12 @@ class AzureML(BackendBase):
             ss = "running"
         elif status in completed:
             ss = "completed"
+        elif status == "Unapproved":
+            #errors.warning("unexpected Azure ML status value: {}".format(status))
+            ss = "queued"
         else:
-            errors.internal_error("unexpected Auzre ML status value: {}".format(status))
+            errors.warning("unexpected Azure ML status value: {}".format(status))
+            ss = "queued"
 
         return ss
 
