@@ -697,6 +697,11 @@ class BackendBase(implements(BackendInterface)):
             self.append(cmds, "nvidia-smi")
 
         self.framework_report(cmds)
+
+        # if log_reports:
+        #     self.append_title(cmds, "POST fw GPU report:", False)
+        #     self.append(cmds, "nvidia-smi")
+
         self.xt_report(cmds)
 
     def add_python_info(self, cmds):
@@ -720,6 +725,14 @@ class BackendBase(implements(BackendInterface)):
         target_cmd = '''python -c "import torch; print('PyTorch:'.ljust(15) + ' ' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"'''
         self.append_package_test(cmds, "torch", target_cmd, "PyTorch")
 
+        # CUDA
+        target_cmd = '''python -c "import torch; print('cuda version:'.ljust(14) + ' ' + torch.version.cuda)"'''
+        self.append_package_test(cmds, "cuda", target_cmd, "cuda")
+
+        # TORCHVISION
+        target_cmd = '''python -c "import torchvision; print('torchvision:'.ljust(15) + ' ' + torchvision.__version__ )"'''
+        self.append_package_test(cmds, "torchvision", target_cmd, "torchvision")
+
         # for now, OMIT tensorflow (can take 30-60 secs to initialize cuda, etc.)
         # # TENSORFLOW
         # target_cmd = '''python -c "import tensorflow as tf; print('Tensorflow:'.ljust(15) + ' ' + tf.__version__ + ', CUDA available: ' + str(tf.test.is_gpu_available()))"'''
@@ -739,6 +752,7 @@ class BackendBase(implements(BackendInterface)):
         pp = args["other_cmds"]
 
         if pp:
+            #console.print("other_cmds: {}".format(pp))
             self.append_title(cmds, "OTHER setup cmds:", False)
             for cmd in pp:
                 self.append(cmds, cmd)
@@ -1032,6 +1046,18 @@ class BackendBase(implements(BackendInterface)):
         else:
             self.append_export(cmds, var_name, '$({})'.format(now_cmd), fix_value=False)
 
+    def append_pre_docker_cmds(self, cmds):
+        if not self.gen_for_windows:
+            self.append(cmds, "hostnamectl")
+
+        # debug nvidia drivers/toolkit combinations
+        self.append(cmds, "nvcc --version")
+        self.append(cmds, "nvidia-smi")
+
+        # TORCH version (is GPU available to torch?)
+        target_cmd = '''python -c "import torch; print('PyTorch:'.ljust(15) + ' ' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"'''
+        self.append_package_test(cmds, "torch", target_cmd, "PyTorch")
+
     def create_wrapper_and_inner(self, cmds, user_cmd, args):
 
         # debug
@@ -1070,6 +1096,7 @@ class BackendBase(implements(BackendInterface)):
                 index=1
 
             self.append_title(docker_cmds, 'START OF XT-LEVEL PROCESSING', double=True, index=index)
+            self.append_pre_docker_cmds(docker_cmds)
 
             self.append_title(docker_cmds, "START DOCKER CONTAINER:")
             self.append_dir(docker_cmds)
@@ -1110,8 +1137,14 @@ class BackendBase(implements(BackendInterface)):
             file_utils.write_text_file(fn_image, full_image_name)
         else:
             # write normal cmds to WRAPPED script
-            insert_index = 1 if self.gen_for_windows else 0
-            self.append_title(cmds, 'START of XT-level processing', double=True, index=insert_index)
+            #insert_index = 1 if self.gen_for_windows else 0
+            pre_cmds = []
+            self.append_title(pre_cmds, 'START of XT-level processing', double=True)
+            self.append_pre_docker_cmds(pre_cmds)
+            if self.gen_for_windows:
+                cmds = [ cmds[0] ] + pre_cmds + cmds[1:]
+            else:
+                cmds = pre_cmds + cmds
 
             fn_wrapped = self.write_cmds_to_file(cmds, bootstrap_dir, self.is_windows, constants.FN_WRAPPED_BAT, constants.FN_WRAPPED_SH)
             utils.copy_to_submit_logs(args, fn_wrapped)
