@@ -669,13 +669,13 @@ class AzureBatch(BackendBase):
 
         network_config = self.create_network_config()
 
-        max_tasks_per_node = None
+        #max_tasks_per_node = None
         enable_inter_node_communication = False
 
         #console.print("is_distributed=", is_distributed)
         
         if is_distributed:
-            max_tasks_per_node = 1
+            #max_tasks_per_node = 1
             enable_inter_node_communication = True
 
         if self.auto_pool:
@@ -704,7 +704,7 @@ class AzureBatch(BackendBase):
                     network_configuration=network_config,
                     enable_auto_scale=True,
                     auto_scale_formula=formula,
-                    max_tasks_per_node = max_tasks_per_node,
+                    #max_tasks_per_node = max_tasks_per_node,
                     enable_inter_node_communication=enable_inter_node_communication)
             else:
                 pool_spec = batch.models.PoolSpecification(
@@ -714,7 +714,7 @@ class AzureBatch(BackendBase):
                     network_configuration=network_config,
                     target_dedicated_nodes=0 if self.use_low_pri else self.num_nodes,
                     target_low_priority_nodes=self.num_nodes if self.use_low_pri else 0,
-                    max_tasks_per_node = max_tasks_per_node,
+                    #max_tasks_per_node = max_tasks_per_node,
                     enable_inter_node_communication=enable_inter_node_communication)
 
             auto_pool = batch.models.AutoPoolSpecification(pool_lifetime_option="job", keep_alive=False, pool=pool_spec)
@@ -727,7 +727,7 @@ class AzureBatch(BackendBase):
                 network_configuration=network_config,
                 target_dedicated_nodes=0 if self.use_low_pri else self.num_nodes,
                 target_low_priority_nodes=self.num_nodes if self.use_low_pri else 0,
-                max_tasks_per_node = max_tasks_per_node,
+                #max_tasks_per_node = max_tasks_per_node,
                 enable_auto_scale=False,                # don't resize to 0 or we won't be able to connect to node
                 enable_inter_node_communication=enable_inter_node_communication)
 
@@ -757,10 +757,12 @@ class AzureBatch(BackendBase):
         
         self.blob_client.create_container(dest_container_name, fail_on_exist=False)
 
+        expire_days = self.config.get("general", "storage-cert-days")
+
         # create an SAS for writing to the container
         sas_token = self.blob_client.generate_container_shared_access_signature(dest_container_name, 
             permission=azureblob.ContainerPermissions.WRITE,
-            expiry=datetime.datetime.utcnow() + datetime.timedelta(hours=24))
+            expiry=datetime.datetime.utcnow() + datetime.timedelta(hours=expire_days*24))
 
         # secret trick: construct SAS URL for the container
         out_container_url = "https://{}.blob.core.windows.net/{}?{}".format(self.store_name, dest_container_name, sas_token)
@@ -1474,7 +1476,7 @@ class AzureBatch(BackendBase):
                 next_offset = start_offset + len(new_bytes)
                 log_source = "live"
 
-            except (batchmodels.BatchErrorException) as ex:
+            except BaseException as ex:
                 # interpret this error as task has terminated
                 console.diag("Exception reading batch log file: {}".format(ex))
 
@@ -1497,7 +1499,7 @@ class AzureBatch(BackendBase):
         elif status in completed:
             ss = "completed"
         else:
-            errors.internal_error("unexpected Azure Batch status value: {}".format(status))
+            errors.warning("unexpected Azure Batch status value: {}".format(status))
 
         return ss
 
