@@ -162,7 +162,7 @@ class BackendInterface(Interface):
 
     # API call
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
-        encoding='utf-8', use_best_log=True):
+        encoding='utf-8', use_best_log=True, log_source=None):
         '''
         Args:
             service_node_info: a service-defined dictionary that identifies a node assoicated with a job
@@ -171,6 +171,7 @@ class BackendInterface(Interface):
             end_offset: last offset of log file to read (if None, rest of file is read)
             encoding: how to encode bytes to text for log content
             use_best_log: when True, backend will continually choose the best log to be return
+            log_source: where log is being read from (None, "live", "after_logs")
         Returns a dictionary with following keys:
             new_text  (the specified subset of the file)
             simple_status (a string, one of: queued, running, completed) 
