@@ -114,10 +114,10 @@ def capture_before_files_zip(store, source_dir, ws_name=None, run_name=None, ext
 
     return copied_files
 
-def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=False, log_events=True):
+def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=False, log_events=True, source="job"):
     files = []
 
-    if job_id:
+    if source == "job":
         # all files contained in JOB BEFORE
         files += store.download_files_from_job(ws_name, job_id, "before/code/**", dest_dir)
         count = len(files)
@@ -125,7 +125,7 @@ def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=Fal
             console.print("  {} CODE files downloaded from store (JOB dir)".format(count))
 
         if log_events:
-            store.log_run_event(ws_name, run_name, "download_before", {"source": "job", "count": count})
+            store.log_run_event(ws_name, run_name, "download_before", {"source": "job", "count": count}, job_id=job_id)
     else:
         if "." in run_name:
             # download from parent's RUN before files
@@ -139,7 +139,7 @@ def download_before_files(store, job_id, ws_name, run_name, dest_dir, silent=Fal
                 store.log_run_event(ws_name, run_name, "download_before", {"source": "parent-run", "count": count})
         
         # download from normal/child RUN before files
-        files += store.download_files_from_run(ws_name, run_name, "before/**", dest_dir)
+        files += store.download_files_from_run(ws_name, run_name, "before/**", dest_dir, job_id=job_id)
         count = len(files)
         if not silent:
             console.print("  {} files downloaded from store (RUN dir)".format(count))
