@@ -362,6 +362,8 @@ class CmdCore():
         # for mirroring files to grok server or storage
         context.mirror_dest = args["mirror_dest"]
         context.mirror_files =  args["mirror_files"]
+        context.mirror_delay_mins =  self.config.get("logging", "mirror-delay-mins")
+        context.show_mirror_calls =  self.config.get("logging", "show-mirror-calls")
         context.grok_server = None   # args["grok_server"]
 
         context.aggregate_dest = args["aggregate_dest"]
@@ -385,6 +387,7 @@ class CmdCore():
         context.db_options = self.config.get("database")
 
         context.shell_launch_prefix = box_info.shell_launch_prefix
+        context.show_missing_jobid_calls = self.config.get("logging", "show-missing-jobid-calls")
         
         #console.print("context=", context)
         return context
