@@ -671,7 +671,7 @@ class XTController(rpyc.Service):
         # create a parent log event for "spawning"
         context = run_info.context
         store = self.store
-        store.log_run_event(context.ws, run_name, "status-change", {"status": "spawning"})  
+        store.log_run_event(context.ws, run_name, "status-change", {"status": "spawning"}, job_id=self.job_id)  
 
         # spawn child run from template
         child_info = self.spawn_child(run_info, run_index, entry)
@@ -748,21 +748,29 @@ class XTController(rpyc.Service):
 
         if context.search_style == "dynamic":
             # perform dynamic HPARAM search
+            log_info("doing dynamic HPARAM search for child", child_name)
             cmd_parts = self.hparam_search.process_child_hparams(child_name, store, context, parent)
         elif self.runsets:
             # select a runset and apply to run
-            runset = self.runsets[run_index % len(self.runsets)]
+            actual_ri = run_index % len(self.runsets)
+            log_info("getting runset for child=" + child_name, "index=" + str(actual_ri))
+
+            runset = self.runsets[actual_ri]
+            log_info("runset for child", runset)
+
             cmd_parts = list(context.cmd_parts)
             cmd_parts = self.hparam_search.apply_runset(runset, cmd_parts, context, store, child_name)
+            log_info("runset applied to cmd_parts", cmd_parts)
         else:
             # select the normal command for the run
+            log_info("normal child without HPARAM search", child_name)
             cmd_parts = context.cmd_parts
 
         # must update context info
         context.run_name = child_name
 
         # log run CMD
-        store.log_run_event(context.ws, child_name, "cmd", {"cmd": cmd_parts, "xt_cmd": context.xt_cmd})  
+        store.log_run_event(context.ws, child_name, "cmd", {"cmd": cmd_parts, "xt_cmd": context.xt_cmd}, job_id=context.job_id)  
 
         # for now, don't log context (contain private credentials and not clear if we really need it)
         # for CHILD runs, record all "context" (from cmd line, user config, default config) in log (for audit/re-run purposes)
@@ -1039,13 +1047,15 @@ class XTController(rpyc.Service):
             self.concurrent = context.concurrent
             log_info("set concurrent", self.concurrent)
 
+        print("context.job_id=", context.job_id)
+
         run_info = RunInfo(run_name, context.ws, cmd_parts, run_script, context.repeat, context, "queued", True, 
             parent_name=None, parent_prep_needed=parent_prep_needed, mirror_close_func = self.stop_mirror_worker, 
             node_id=self.node_id, run_index=None, store=self.store, max_delay=context.max_delay)
 
         # log run QUEUED event 
         store = self.store
-        store.log_run_event(context.ws, run_name, "queued", {}) 
+        store.log_run_event(context.ws, run_name, "queued", {}, job_id=context.job_id) 
 
         # queue job to be run
         with queue_lock:
@@ -1159,7 +1169,7 @@ class XTController(rpyc.Service):
         os.system("ls -lt {} | grep -vh '^total' | head -n 30".format(rundir))
 
         # HP search generated config.yaml is stored in RUN BEFORE files
-        files = capture.download_before_files(store, None, context.ws, run_name, dest_dir=rundir, silent=True)
+        files = capture.download_before_files(store, context.job_id, context.ws, run_name, dest_dir=rundir, silent=True, source="run")
         log_info("downloaded", "{} BEFORE files from RUN STORE".format(len(files)))
 
         run_script = run_info.run_script
@@ -1304,7 +1314,7 @@ class XTController(rpyc.Service):
             log_title("CHILD RESTART detected")
 
         start_event_name = "restarted" if context.restart else "started"
-        store.log_run_event(context.ws, run_name, start_event_name, {})  
+        store.log_run_event(context.ws, run_name, start_event_name, {}, job_id=job_id)  
         #prep_script = run_info.prep_script  
 
         if not job_id in self.running_jobs:
@@ -1357,10 +1367,13 @@ class XTController(rpyc.Service):
         log_info("process created", p)
 
     def start_mirror_worker(self, store, run_info, rundir, run_name, context):
-        log_info("starting MIRROR thread", context.grok_server)
+        log_info("starting MIRROR worker for run", run_name)
         log_info("mirror-dest", context.mirror_dest)
         log_info("mirror-path", context.mirror_files)
-        worker = MirrorWorker(store, rundir, context.mirror_dest, context.mirror_files, context.grok_server, context.ws, run_name)
+
+        worker = MirrorWorker(store, rundir, context.mirror_dest, context.mirror_files, context.grok_server, context.ws, run_name, 
+            job_id=context.job_id, mirror_delay_mins=context.mirror_delay_mins, 
+            show_calls=context.show_mirror_calls)
         
         run_info.mirror_worker = worker
         self.mirror_workers.append(worker)
@@ -1368,6 +1381,8 @@ class XTController(rpyc.Service):
         worker.start()
 
     def stop_mirror_worker(self, run_info):
+        log_info("stop_mirror_worker: run_info.mirror_worker", run_info.mirror_worker)
+        
         if run_info.mirror_worker:
             worker = run_info.mirror_worker
             worker.stop()
