@@ -40,7 +40,7 @@ def get_command_dicts(prev_exper, curr_exper, browse_flag, browse_opt, timeout_o
 
 	    # RUNS
 	    {"title": "run script on LOCAL MACHINE", "cmd": "xt run {}--target=local --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
-	    {"title": "run script on PHILLY", "cmd": "xt run {}--target=philly --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
+	    {"title": "run script on ITP-US", "cmd": "xt run {}--target=itp-us --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
 	    {"title": "run script on AZURE BATCH", "cmd": "xt run {}--target=batch --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
 	    {"title": "run script on AZURE ML", "cmd": "xt run {}--target=aml --exper={} code\miniMnist.py".format(monitor_opt, curr_exper)},
 
@@ -70,7 +70,7 @@ def get_command_dicts(prev_exper, curr_exper, browse_flag, browse_opt, timeout_o
         },
 
 	    # VIEW PORTAL
-	    {"title": "Browse the portal for the 'philly' target", "cmd": "xt view portal philly {}".format(browse_flag)},
+	    {"title": "Browse the portal for the 'itp' target", "cmd": "xt view portal itp {}".format(browse_flag)},
 
 	    {"title": "Browse the portal for the 'aml' target", "cmd": "xt view portal aml {}".format(browse_flag)},
 
@@ -94,8 +94,8 @@ def get_command_dicts(prev_exper, curr_exper, browse_flag, browse_opt, timeout_o
 
 	    # MOUNT data and DOWNLOAD model
 	    {
-            "title": "run script, mounting data and downloading model for eval", 
-            "cmd": "xt run {}--target=philly --data-action=mount --model-action=download code\miniMnist.py --auto-download=0 --eval-model=1". \
+            "title": "run script on itp-us, mounting data and downloading model for eval", 
+            "cmd": "xt run {}--target=itp-us --data-action=mount --model-action=download code\miniMnist.py --auto-download=0 --eval-model=1". \
                 format(monitor_opt)
         },
 
