@@ -29,7 +29,13 @@ cmd_count = 0
 
 ARCHIVES_DIR = "..\\xt_demo_archives"
 
-def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=None):
+def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=None, 
+    store=None, readonly_ws="xt-demo"):
+
+    global cmds, cmd_count
+    cmds = []
+    cmd_count = 0
+
     config = xt_config.get_merged_config()
 
     if basic_mode is None:
@@ -46,6 +52,7 @@ def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=Non
     # SET THESE before each demo (exper24 should be a multi-service set of simple runs)
     prev_exper = "exper18"
     curr_exper = "exper26"
+    big_job = "xtd_job10608" if store.endswith("v1") else "job10608"
 
     if basic_mode:
         command_dicts = commands_basic.get_command_dicts(
@@ -56,7 +63,8 @@ def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=Non
             timeout_opt,
             templ,
             ARCHIVES_DIR, 
-            monitor_opt)
+            monitor_opt, 
+            readonly_ws)
     else:
         command_dicts = commands_advanced.get_command_dicts(
             prev_exper,
@@ -66,7 +74,9 @@ def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=Non
             timeout_opt,
             templ,
             ARCHIVES_DIR,
-            monitor_opt)
+            monitor_opt,
+            big_job,
+            readonly_ws)
 
     if not has_gui:
         command_dicts = list(filter(
@@ -79,7 +89,7 @@ def build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=1, basic_mode=Non
             command_dicts)
 
     list(map(
-        lambda cmd_dict: add_cmd(cmd_dict["title"], cmd_dict["xt_cmd"]),
+        lambda cmd_dict: add_cmd(cmd_dict["title"], cmd_dict["cmd"]),
         command_dicts
     ))
 
@@ -89,7 +99,7 @@ def add_cmd(title=None, xt_cmd=None, silent=False):
     title = file_utils.fix_slashes(title)
     xt_cmd = file_utils.fix_slashes(xt_cmd)
 
-    cmd = {"title": title, "xt_cmd": xt_cmd, "silent": silent}
+    cmd = {"title": title, "cmd": xt_cmd, "silent": silent}
     cmds.append(cmd)
 
 def wait_for_nav_key(cmd_text, auto_mode):
@@ -152,7 +162,7 @@ def navigate(cmds, auto_mode, steps):
 
         # show title, cmd
         cmd = cmds[index]
-        cmd_text = cmd["xt_cmd"]
+        cmd_text = cmd["cmd"]
         silent = cmd["silent"]
 
         if silent:
@@ -240,6 +250,8 @@ def parse_args(arg_list=None):
     parser.add_argument('--philly', default=1, type=int, help='should Philly tests be run')
     parser.add_argument('--basic_mode', default=None, type=int, help='should Philly tests be run')
     parser.add_argument('--quick_test', default=0, type=int, help='specifies we are running as part of the quick-test')
+    parser.add_argument('--store', type=str, help='the name of the store being used in config file')
+    parser.add_argument('--readonly_ws', type=str, help='the name of the readonly workspace to use for previous runs')
     parser.add_argument("steps", nargs="*", help="the steps to be run")
 
     if not arg_list:
@@ -286,8 +298,10 @@ def main(arg_list=None):
     quick_test = args.quick_test
     philly = args.philly
     basic_mode = args.basic_mode
+    readonly_ws = args.readonly_ws
 
-    build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=philly, basic_mode=basic_mode)
+    build_cmds(auto_mode, quick_test, nomonitor, nogui, philly=philly, basic_mode=basic_mode, 
+        store=args.store, readonly_ws=readonly_ws)
 
     steps = parse_steps(args.steps)
     run_tests = True
