@@ -2,8 +2,8 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT license.
 #
-# eval_hp_search.py: evaluates runs to-date from specified hyperparmeter search (job or experiment name)
-# runnable standalone or from XT (xt plot 
+# eval_hp_search.py: evaluates runs to-date from specified hyperparameter search (job or experiment name)
+# runnable standalone or from XT (xt plot summary)
 
 import os
 import math
@@ -24,6 +24,7 @@ from xtlib.hparams import hp_helper
 from xtlib.storage.store import Store
 from xtlib.helpers.xt_config import get_merged_config
 
+
 plt = None     # JIT import to keep XT from always loading matplotlib (and font cache building)
 
 
@@ -37,7 +38,7 @@ def plot(xs, ys, ds, job_name, y_axis_label):
     ''' Displays a plot of the hyperparameter search progress. '''
     title = 'Retrospective evaluation of the best HP combination after each run'
     fig = plt.figure(figsize=(12, 8))
-    fig.canvas.set_window_title('scripts\eval_hp_search.py')
+    fig.canvas.manager.set_window_title('xt plot summary')
     perf_axes = fig.add_axes([0.08, 0.09, 0.88, 0.84])
     perf_axes.set_title(title, fontsize=16)
     ymax = ys + ds
@@ -51,19 +52,16 @@ def plot(xs, ys, ds, job_name, y_axis_label):
     plt.show()
 
 
-def get_time(run):
-    return run.time
+def get_end_id(run):
+    return run.end_id
 
-def get_neg_run_mean(run):
-    return -run.mean
+def get_neg_run_value(run):
+    return -run.value
 
 class RunSet():
     ''' Encapsulates a list of runs that share the same HP combination. '''
-    def __init__(self, history, hp_id__setting_id__list, config_str):
-        # sort to ensure a consistent order
-        if hp_id__setting_id__list:
-            hp_id__setting_id__list.sort()
-
+    def __init__(self, id, history, hp_id__setting_id__list, config_str):
+        self.id = id
         self.history = history
         self.hp_id__setting_id__list = hp_id__setting_id__list
         self.config_str = config_str
@@ -146,7 +144,7 @@ class RunSet():
         elif self.post_var > other.post_var:
             return False
         # Older is better.
-        return self.runs[0].time < other.runs[0].time
+        return self.runs[0].end_id < other.runs[0].end_id
 
     def should_move_up(self):
         # True if the runset above this one belongs below it.
@@ -173,29 +171,38 @@ class RunSet():
         self.dn = self.dn.dn
 
     def report_configuration(self):
-        # print("    config={}".format(self.config_str))
         i = 0
         for hp in self.history.hparams:
-            if hp.in_hp_section and hp.has_multiple_values:
-                assert hp.id == self.hp_id__setting_id__list[i][0]
-                console.print("{} = {}".format(hp.name, hp.settings[self.hp_id__setting_id__list[i][1]].value))
-                i += 1
+            assert hp.id == self.hp_id__setting_id__list[i][0]
+            console.print("{} = {}".format(hp.name, hp.settings[self.hp_id__setting_id__list[i][1]].value))
+            i += 1
 
     def report_configuration_string(self):
         i = 0
         sz = ''
         for hp in self.history.hparams:
-            if hp.in_hp_section and hp.has_multiple_values:
-                width = max(MIN_COL_WID, len(hp.name))
-                format_string = ' {' + ':{}s'.format(width) + '}'
-                sz += format_string.format(str(hp.settings[self.hp_id__setting_id__list[i][1]].value))
-                i += 1
+            width = max(MIN_COL_WID, len(hp.name))
+            format_string = ' {' + ':{}s'.format(width) + '}'
+
+            try:
+                setting_index = ""
+                setting_index = self.hp_id__setting_id__list[i][1]
+                val = hp.settings[setting_index]
+                txt = str(val.value)
+            except BaseException as ex:
+                txt = "[{}]?".format(setting_index)
+            sz += format_string.format(txt)
+            i += 1
         return sz
 
+    def report(self, title):
+        fixed_values = '   {}   {}             {:9.3f}'.format(self.id, len(self.runs), self.post_mean)
+        console.print('     ' + self.report_configuration_string() + fixed_values)
+
     def add_run(self, run):
         ''' Welford's online algorithm for single-pass computation of variance. '''
         self.runs.append(run)
-        value = run.mean
+        value = run.value
         self.sample_count += 1
         delta = value - self.sample_mean
         self.sample_mean += delta / self.sample_count
@@ -219,32 +226,58 @@ class RunSetTrophy():
 
 
 class Run():
-    def __init__(self, run_summary, metric_name):
+    def __init__(self, run_summary, primary_metric, verbose=False, use_azure_storage=False):
         ''' Maintains the per-run information. '''
-        self.hpname_hpvalue_list = []
-        self.num_reports = 0
-        self.mean = 0.
-        self.time = None
-        json_line = json.loads(run_summary)
-        event_dict_list = json_line['log']
-        for event_dict in event_dict_list:
-            event_type = event_dict["event"]
-            if event_type == "hparams":
-                hp_data = event_dict["data"]
-                for hpname in hp_data:
-                    self.hpname_hpvalue_list.append((hpname, hp_data[hpname]))
-            elif event_type == "metrics":
-                metric_dict = event_dict["data"]
-                if metric_name in metric_dict:
-                    metric_value = float(metric_dict[metric_name])
-                    self.mean += metric_value
-                    self.num_reports += 1
-            elif event_type == "created":
-                run_data = event_dict["data"]
-                self.run_name = run_data["run_name"]
-            self.time = event_dict["time"]  # The time of the last event of the run.
-        if self.num_reports > 0:
-            self.mean /= self.num_reports
+        if use_azure_storage:
+            # run_summary comes from Azure Storage.
+            self.run_name = None
+            self.hpname_hpvalue_list = []
+            self.skip = True
+            self.value = 0.
+            self.end_id = None
+            json_line = json.loads(run_summary)
+            event_dict_list = json_line['log']
+            for event_dict in event_dict_list:
+                event_type = event_dict["event"]
+                if event_type == "hparams":
+                    hp_data = event_dict["data"]
+                    for hpname in hp_data:
+                        self.hpname_hpvalue_list.append((hpname, hp_data[hpname]))
+                elif event_type == "metrics":
+                    metric_dict = event_dict["data"]
+                    if primary_metric in metric_dict:
+                        self.value += float(metric_dict[primary_metric])
+                        self.skip = False
+                elif event_type == "created":
+                    if self.run_name is not None:
+                        # There was a previous "created" event, so apparently the run was restarted.
+                        if verbose:
+                            console.print('Unexpected created event in {}. Resetting the run.'.format(self.run_name))
+                        # Reset all fields.
+                        self.run_name = None
+                        self.hpname_hpvalue_list = []
+                        self.skip = True
+                        self.value = 0.
+                        self.end_id = None
+                    run_data = event_dict["data"]
+                    self.run_name = run_data["run_name"]
+                self.end_id = event_dict["time"]  # The time of the last event of the run.
+            if self.skip and verbose:
+                console.print("Skipping {}. (Missing {} metric report.)".format(self.run_name, primary_metric))
+        else:
+            # run_summary comes from the database.
+            self.skip = False
+            self.value = 0.
+            self.run_name = run_summary['run_name']
+            self.end_id = run_summary['end_id']
+            metrics = run_summary['metrics']
+            if primary_metric in metrics:
+                self.value = float(metrics[primary_metric])
+                self.hparams = run_summary['hparams']
+            else:
+                self.skip = True
+                if verbose:
+                    console.print("Skipping {}. (Missing {} metric report.)".format(self.run_name, primary_metric))
 
 
 class HyperparameterSetting():
@@ -257,57 +290,25 @@ class HyperparameterSetting():
 
 class Hyperparameter():
     ''' Stores the name and settings for a single hyperparameter. '''
-    def __init__(self, id, name, value_string, value_list, in_hp_section):
+    def __init__(self, id, name, value_list):
         self.id = id
         self.name = name
-        self.in_hp_section = in_hp_section
         self.value_setting_dict = {}
         self.settings = []
-        self.single_value = None
-        self.has_multiple_values = False
-        if value_string:
-            value_strs = value_string.split(',')
-            values = []
-            for value_str in value_strs:
-                values.append(self.cast_value(value_str.strip()))
-            values.sort()
-        elif value_list:
-            values = value_list
-        else:
-            assert False, 'Either value_string or value_list must be provided.'
-        for value in values:
+        for value in value_list:
             self.add_setting(value)
-        self.has_multiple_values = (len(self.settings) > 1)
-
-    def cast_value(self, value_str):
-        if value_str == 'None':
-            new_value = None
-        elif value_str == 'True':
-            new_value = True
-        elif value_str == 'False':
-            new_value = False
-        else:
-            try:
-                new_value = int(value_str)
-            except ValueError:
-                try:
-                    new_value = float(value_str)
-                except ValueError:
-                    new_value = value_str
-        return new_value
 
     def add_setting(self, value):
         assert value not in self.value_setting_dict.keys()
         setting = HyperparameterSetting(len(self.settings), self, value)
         self.value_setting_dict[value] = setting
         self.settings.append(setting)
-        self.single_value = value
 
 
 class SearchHistory():
     ''' Loads, processes, and evaluates the runs of a single hyperparameter search job. '''
     def __init__(self, args=None, agg_name=None, workspace=None, timeout=None, primary_metric=None, 
-        hp_config_file_name=None, max_workers=1, xt_config=None):
+        hp_config_file_name=None, max_workers=1, max_runs=0, xt_config=None, verbose=False, use_azure_storage=False):
 
         global plt
         import matplotlib.pyplot as plt
@@ -317,9 +318,11 @@ class SearchHistory():
             args = parse_args(arg_list)
 
             # set args from XT
-            args.metric = primary_metric
+            args.primary_metric = primary_metric
             args.hp_def_file = os.path.basename(hp_config_file_name)
-            args.num_reports_per_run = None
+            args.max_runs = max_runs
+            args.verbose = verbose
+            args.use_azure_storage = use_azure_storage
 
         self.args = args
         self.timeout = timeout
@@ -344,29 +347,20 @@ class SearchHistory():
 
     def load_hp_definitions(self):
         ''' Get the hyperparameter definitions, downloaded from Azure if necessary. '''
-        if self.args.hp_def_file == 'config.txt':
-            config_path = self.DIR_PATH + args.hp_def_file
-            self.define_hyperparameters_from_config_txt(config_path)
-        else:
-            hp_config_path = self.DIR_PATH + self.args.hp_def_file
-            if not exists(hp_config_path):
-                self.init_xtstore()
+        hp_config_path = self.DIR_PATH + self.args.hp_def_file
+        if not exists(hp_config_path):
+            self.init_xtstore()
+            job_path = search_helper.get_hp_config_path(self.xtstore, self.ws_name, "job", self.args.job)
+            self.xtstore.download_file_from_job(self.ws_name, self.args.job, job_path, hp_config_path)
+        self.define_hyperparameters_from_config_yaml(hp_config_path)
 
-                #job_path = constants.HP_CONFIG_DIR + self.args.hp_def_file
-                job_path = search_helper.get_hp_config_path(self.xtstore, self.ws_name, "job", self.args.job)
-
-                self.xtstore.download_file_from_job(self.ws_name, self.args.job, job_path, hp_config_path)
-            self.define_hyperparameters_from_config_yaml(hp_config_path)
-
-    def load_runs(self):
+    def load_runs_from_storage(self):
         ''' Download the runs from Azure. '''
         runs_file_name = constants.ALL_RUNS_FN
         all_runs_file_path = self.DIR_PATH + runs_file_name
 
         if not self.args.no_download:
             self.init_xtstore()
-
-            #console.print("  downloading all_runs.jsonl file...", end="", flush=True)
             started = time.time()
 
             nodes_read = self.xtstore.download_all_runs_blobs(self.ws_name, "job", self.args.job, 
@@ -378,14 +372,10 @@ class SearchHistory():
         run_summaries = open(all_runs_file_path, 'r')
         self.num_incomplete_runs = 0
         for run_summary in run_summaries:
-            run = Run(run_summary, self.args.metric)
+            run = Run(run_summary, self.args.primary_metric, verbose=self.args.verbose, use_azure_storage=True)
             skip_this_run = False
 
-            if run.num_reports == 0:  # Parent runs report no metrics.
-                continue
-
-            if self.args.num_reports_per_run and run.num_reports != self.args.num_reports_per_run:
-                self.num_incomplete_runs += 1
+            if run.skip:
                 continue
 
             # Try to assemble a configuration string for this run.
@@ -404,16 +394,14 @@ class SearchHistory():
 
                 hparam = self.name_hparam_dict[hp_name]
                 
-                if not hparam.in_hp_section:  # Ignore controls.
-                    continue
-
                 if hp_value not in hparam.value_setting_dict:
+                    if self.args.verbose:
+                        console.print("Skipping {}  (Unexpected hparam value {}:{})".format(run.run_name, hparam.name, hp_value))
                     skip_this_run = True
                     break  # Skip this run. Its value must have been removed from config.txt.
 
-                if hparam.has_multiple_values:  # Filter to HPs with multiple values.
-                    setting = hparam.value_setting_dict[hp_value]
-                    hp_id__setting_id__list.append((hparam.id, setting.id))
+                setting = hparam.value_setting_dict[hp_value]
+                hp_id__setting_id__list.append((hparam.id, setting.id))
 
             if skip_this_run:
                 continue
@@ -423,34 +411,103 @@ class SearchHistory():
             run.config_str = str(hp_id__setting_id__list)
             self.runs.append(run)
 
-        self.runs.sort(key=get_time)
+        self.runs.sort(key=get_end_id)
 
-    def define_hyperparameters_from_config_txt(self, config_path):
-        ''' To enable reading of ancient jobs. '''
-        self.name_hparam_dict = {}
-        self.hparams = []
-        with open(config_path, 'r') as file:
-            hp_config_file_text = file.read()
-        line_num = 0
-        in_hp_section = False
-        lines = hp_config_file_text.split('\n')
-        for line in lines:
-            line_num += 1
-            if len(line) == 0:
+    def load_runs_from_db(self):
+        # Get the run summaries.
+        self.init_xtstore()
+        filter_dict = {"job_id":self.args.job, "end_id":{"$gt": 0}, 'status':'completed', "metrics.hpmax":{"$exists": True}, 'is_parent':False}
+        fields_dict = {'run_name':1, 'end_id':1, 'hparams':1, 'metrics.hpmax':1}
+        run_summaries = self.xtstore.get_all_runs("job", self.ws_name, self.args.job, filter_dict, fields_dict, use_cache=False)
+
+        # Create runs from the summaries.
+        for run_summary in run_summaries:
+            run = Run(run_summary, self.args.primary_metric, verbose=self.args.verbose)
+            if run.skip:
                 continue
-            if line[0] == '#':
-                if "CONTROLS" in line:
-                    in_hp_section = False
-                elif "HYPERPARAMETERS" in line:
-                    in_hp_section = True
+
+            # Gather the expected hparams from this run's report.
+            search_keys = list(self.name_hparam_dict.keys())
+            run.hpname_hpvalue_list = []
+            for hp_name in search_keys:
+                if hp_name in run.hparams:
+                    run.hpname_hpvalue_list.append((hp_name, run.hparams[hp_name]))
+                else:
+                    run.skip = True  # Some hparam was not found in the report.
+                    if self.args.verbose:
+                        console.print("Skipping {}  (missing hparam={})".format(run.run_name, hp_name))
+                    break
+            if run.skip:
                 continue
-            halves = line.split('=')
-            name_string, value_string = halves[0].strip(), halves[1].strip()
-            if '#' in value_string:
-                value_string = value_string[:value_string.index('#')].strip()
-            hp = Hyperparameter(len(self.hparams), name_string, value_string, None, in_hp_section)
-            self.hparams.append(hp)
-            self.name_hparam_dict[name_string] = hp
+
+            # Try to assemble a configuration string for this run.
+            hp_id__setting_id__list = []
+            for hp_name, hp_value in run.hpname_hpvalue_list:
+                hparam = self.name_hparam_dict[hp_name]
+                if hp_value not in hparam.value_setting_dict:
+                    if self.args.verbose:
+                        console.print("Skipping {}  (Unexpected hparam value {}:{})".format(run.run_name, hparam.name, hp_value))
+                    run.skip = True  # Unexpected hparam value.
+                    break
+                setting = hparam.value_setting_dict[hp_value]
+                hp_id__setting_id__list.append((hparam.id, setting.id))
+            if run.skip:
+                continue
+
+            # Add this as a valid run
+            run.hp_id__setting_id__list = hp_id__setting_id__list
+            run.config_str = str(hp_id__setting_id__list)
+            self.runs.append(run)
+
+        # Sort the runs by order of completion.
+        self.runs.sort(key=get_end_id)
+        for i, run in enumerate(self.runs):
+            assert isinstance(run.end_id, int)
+            if i > 0:
+                assert run.end_id > self.runs[i-1].end_id
+
+    def populate_runsets(self, global_runset):
+        ''' Groups runs into runsets that share the same HP combination. '''
+        prior_mean = global_runset.sample_mean
+        prior_var = global_runset.get_sample_var()
+        best_runset = global_runset
+        if self.args.verbose:
+            console.print("\nDetailed history of trophy holders...")
+        for i, run in enumerate(self.runs):
+            config_str = run.config_str
+            hp_id__setting_id__list = run.hp_id__setting_id__list
+            if config_str not in self.configstr_runset_dict.keys():
+                runset = RunSet(len(self.runsets), self, hp_id__setting_id__list, config_str)
+                self.configstr_runset_dict[config_str] = runset
+                self.runsets.append(runset)
+            runset = self.configstr_runset_dict[config_str]
+            runset.add_run(run)
+            runset.update_posteriors(prior_mean, prior_var)
+            if not runset.in_list:
+                runset.insert(global_runset.dn, global_runset)  # Top of the list.
+            runset.reposition()
+            if global_runset.dn != best_runset:
+                best_runset = global_runset.dn
+                self.best_runset_trophies.append(RunSetTrophy(i, best_runset))
+                if self.args.verbose:
+                    console.print("New trophy goes to runset {}".format(best_runset.id))
+                    if best_runset.post_mean is None:
+                        console.print("    HPMAX = None")
+                    else:
+                        console.print("    HPMAX = {:9.3f}".format(best_runset.post_mean))
+                    if best_runset.dn is not None:
+                        if best_runset.dn.post_mean is None:
+                            console.print("    prev  = None")
+                        else:
+                            console.print("    prev  = {:9.3f}".format(best_runset.dn.post_mean))
+                    for r in best_runset.runs:
+                        console.print('    {}'.format(r.run_name))
+            else:
+                self.best_runset_trophies[-1].num_runs += 1
+        if self.args.verbose:
+            console.print("\nAll runsets...")
+            for runset in self.runsets:
+                runset.report(" {:5d}  ".format(runset.id))
 
     def define_hyperparameters_from_config_yaml(self, hp_config):
         '''
@@ -486,47 +543,20 @@ class SearchHistory():
                 if omit_value:
                     continue
 
-            hp = Hyperparameter(len(self.hparams), hpname, None, valuelist, True)
+            hp = Hyperparameter(len(self.hparams), hpname, valuelist)
             self.hparams.append(hp)
             self.name_hparam_dict[hpname] = hp
 
-    def populate_runsets(self, global_runset):
-        ''' Groups runs into runsets that share the same HP combination. '''
-        prior_mean = global_runset.sample_mean
-        prior_var = global_runset.get_sample_var()
-        best_runset = global_runset
-        num_trophies = 0
-        for i, run in enumerate(self.runs):
-            config_str = run.config_str
-            hp_id__setting_id__list = run.hp_id__setting_id__list
-            if config_str not in self.configstr_runset_dict.keys():
-                runset = RunSet(self, hp_id__setting_id__list, config_str)
-                self.configstr_runset_dict[config_str] = runset
-                self.runsets.append(runset)
-            runset = self.configstr_runset_dict[config_str]
-            runset.add_run(run)
-            runset.update_posteriors(prior_mean, prior_var)
-            if not runset.in_list:
-                runset.insert(global_runset.dn, global_runset)  # Top of the list.
-            runset.reposition()
-            if global_runset.dn != best_runset:
-                best_runset = global_runset.dn
-                num_trophies += 1
-                self.best_runset_trophies.append(RunSetTrophy(i, best_runset))
-            else:
-                self.best_runset_trophies[-1].num_runs += 1
-        # print("{} trophies".format(num_trophies))
-
     def evaluate(self):
         # Get the hyperparameters and the runs.
         self.load_hp_definitions()
-        self.load_runs()
-
-        console.print("  analyzing runs...", end="", flush=True)
-        started = time.time()
+        if self.args.use_azure_storage:
+            self.load_runs_from_storage()
+        else:
+            self.load_runs_from_db()
 
         # Add the runs to a global runset, which will double as the sentinel of the runset list.
-        global_runset = RunSet(None, None, None)
+        global_runset = RunSet(0, None, None, None)
         for run in self.runs:
             global_runset.add_run(run)
         global_runset.sent = True
@@ -536,6 +566,12 @@ class SearchHistory():
         if self.args.max_runs and (self.args.max_runs < len(self.runs)):
             self.runs = self.runs[0:self.args.max_runs]
 
+        if self.args.verbose:
+            if self.args.verbose:
+                console.print("\nAll runs in order of completion...")
+            for run in self.runs:
+                console.print('{}  {:9.3f}'.format(run.run_name, run.value))
+
         # Populate the runset list.
         self.populate_runsets(global_runset)
 
@@ -550,24 +586,20 @@ class SearchHistory():
         #build headers and rows for report
         hp_names = []
         for hp in self.hparams:
-            if hp.in_hp_section and hp.has_multiple_values:
-                hp_names.append(hp.name)
+            hp_names.append(hp.name)
 
-        fixed_names = ['BEST FOR', 'EST HPMAX', 'RUNS', 'BEST']
+        fixed_names = ['RUNSET', 'SIZE', 'BEST_FOR', 'EST_HPMAX', 'BEST_RUN']
         avail_cols = hp_names + fixed_names
 
         records = []
         for trophy in self.best_runset_trophies:
-            trophy.runset.runs.sort(key=get_neg_run_mean)
+            trophy.runset.runs.sort(key=get_neg_run_value)
             best = '{}'.format(trophy.runset.runs[0].run_name)  # To show only the best run for each runset.
 
-            # print(" {}{:3d} runs   {:9.6f}  {:3d}   {}".format(trophy.runset.report_configuration_string(),
-            #     trophy.num_runs, trophy.runset.post_mean, len(trophy.runset.runs), best))
-
             hp_values = trophy.runset.report_configuration_string().split()
             hd = {header:utils.make_numeric_if_possible(value) for header, value in zip(hp_names, hp_values)}
 
-            fixed_values = [trophy.num_runs, trophy.runset.post_mean, len(trophy.runset.runs), best]
+            fixed_values = [trophy.runset.id, len(trophy.runset.runs), trophy.num_runs, trophy.runset.post_mean, best]
             fd = {name: value for name, value in zip(fixed_names, fixed_values)}
 
             # merge fd into hd
@@ -584,9 +616,6 @@ class SearchHistory():
                 ds[x] = d
                 x += 1
 
-        elapsed = time.time() - started
-        console.print(" (elapsed: {:.2f}".format(elapsed))
-
         from xtlib.report_builder import ReportBuilder   
         builder = ReportBuilder(self.xt_config, self.xtstore)
 
@@ -595,13 +624,13 @@ class SearchHistory():
             report_type="run-reports", max_col_width=50)
 
         # print report
-        print()
-        print(text)
+        console.print()
+        console.print(text)
 
-        print("  History of the best hyperparameter combination after each of the {} completed runs.".format(len(self.runs)))
+        console.print("The table above is the history of the best hyperparameter combination (set of runs) after each of the {} completed runs.".format(len(self.runs)))
 
         # Plot the search progress.
-        y_axis_label = "Posterior estimate of {}".format(self.args.metric)
+        y_axis_label = "Posterior estimate of {}".format(self.args.primary_metric)
         plot(xs, ys, ds, self.args.job, y_axis_label)
 
 
@@ -615,9 +644,10 @@ def parse_args(args_list=None):
     parser.add_argument('--job_dir', action='store', help='Path to the parent directory of the given job, where files will be stored. Defaults to dilbert/jobs/', default='../jobs')
     parser.add_argument('--max_runs', action='store', help='Terminate the evaluation after this many runs.', type=int, default=0)
     parser.add_argument('--metric', action='store', help='(For old jobs) Name of the metric maximized by HP tuning. Defaults to hpmax.', default='hpmax')
-    parser.add_argument('--num_reports_per_run', action='store', help='(For old jobs) Expected number of metric reports in each run.', type=int, default=1)
     parser.add_argument('--hp_def_file', action='store', help='(For old jobs) Name of the file defining the HPs.', default='uploaded_hp_config.yaml')
     parser.add_argument('--workspace', action='store', help='name of the workspace for the job', default='')
+    parser.add_argument('--verbose', action='store_true', help='Print extra output to the console for debugging.')
+    parser.add_argument('--use_azure_storage', action='store_true', help='Pulls run results from Azure Storage instead of SQL DB (the default).')
     return parser.parse_args(args_list)
 
 
