@@ -504,7 +504,7 @@ class ImplCompute(ImplBase):
         tmp_dir = file_utils.make_tmp_dir("rerun_cmd")
         job_id = self.store.get_job_id_of_run(workspace, run_name)
         capture.download_before_files(self.store, job_id, workspace, run_name, tmp_dir, 
-            silent=True, log_events=False)
+            silent=True, log_events=False, source="run")
          
         # move to tmp_dir so files get captured correctly
         prev_cwd = os.getcwd()
