@@ -434,14 +434,19 @@ class PlotBuilder():
 
     def get_agg_df(self, df, agg_op, df_cols):
         agg_dict = {}
+        df_out = None
 
         for col in self.col_names:
             if col in df_cols:
                 agg_dict[col] = agg_op
 
-        df_out = df.agg(agg_dict)  
-        #df3 = df2.fillna(method='ffill')
-        df_out = df_out.reset_index()
+        # if not agg_dict:
+        #     errors.config_error("Some columns not found: {}".format(self.col_names))
+
+        if agg_dict:
+            df_out = df.agg(agg_dict)  
+            #df3 = df2.fillna(method='ffill')
+            df_out = df_out.reset_index()
 
         return df_out
 
@@ -504,15 +509,18 @@ class PlotBuilder():
 
     def build_agg_stat(self, df_agg_from, stat, df_cols, dfx):
         df_stat = self.get_agg_df(df_agg_from, stat, df_cols)
-        stat_name = "_{}_".format(stat.upper())
 
-        for col in self.col_names:
-            if col in df_stat.columns:
-                # extract stat data for col
-                stat_data = df_stat[col]
+        # not all data sets in job must be used, so allow some to be empty
+        if df_stat is not None:
+            stat_name = "_{}_".format(stat.upper())
 
-                # add col data as new name to dfx
-                dfx[col + stat_name] = stat_data
+            for col in self.col_names:
+                if col in df_stat.columns:
+                    # extract stat data for col
+                    stat_data = df_stat[col]
+
+                    # add col data as new name to dfx
+                    dfx[col + stat_name] = stat_data
 
         return dfx
 
@@ -598,7 +606,8 @@ class PlotBuilder():
         '''
 
         for cols, df in data_frames_by_cols.items():
-            if y_col in df.columns:
+            #if y_col in df.columns:
+            if df is not None and y_col in df.columns:  
                 
                 # filter values for specified run name
                 group_indexes = df[self.group_by]==group_name 
