@@ -185,7 +185,8 @@ def root(help=""):
     return decorator_root
 
 # COMMAND decorator processor
-def command(name=None, group=None, kwgroup=None, kwhelp=None, options_before_args=False, keyword_optional=False, pass_by_args=False, help=""):
+def command(name=None, group=None, kwgroup=None, user_filters=False, kwhelp=None, 
+    options_before_args=False, keyword_optional=False, pass_by_args=False, help=""):
     '''
     builds a nested dictionary of name parts for multi-word commands 
     and their associated functions.
@@ -219,7 +220,7 @@ def command(name=None, group=None, kwgroup=None, kwhelp=None, options_before_arg
 
         cmd_info =  {"name": cmd_name, "options_before_args": options_before_args, "keyword_optional": keyword_optional, "pass_by_args": pass_by_args, 
             "group": group, "func": func, "arguments": [], "options": [], "examples": [], "faqs": [], "hidden": False, "see_alsos": [],
-            "kwgroup": kwgroup, "kwhelp": kwhelp, "help": help}
+            "kwgroup": kwgroup, "kwhelp": kwhelp, "help": help, "user_filters": user_filters}
 
         dd[""] = cmd_info
 
@@ -374,7 +375,7 @@ def example(text, task="", image=None):
             return func(*args, **kwargs)
 
         if debug_decorators:
-            console.print("example decorator called, name=", name, ", func=", func.__name__)
+            console.print("example decorator called, text={}, func={}".format(text, func.__name__))
 
         global current_cmd_info, root_cmd_info
 
@@ -397,7 +398,7 @@ def see_also(text, page_path=""):
             return func(*args, **kwargs)
 
         if debug_decorators:
-            console.print("see_also decorator called, name=", name, ", func=", func.__name__)
+            console.print("see_also decorator called, text={}, func=".format(text, func.__name__))
 
         global current_cmd_info, root_cmd_info
 
@@ -420,7 +421,7 @@ def faq(question, answer):
             return func(*args, **kwargs)
 
         if debug_decorators:
-            console.print("faq decorator called, name=", name, ", func=", func.__name__)
+            console.print("faq decorator called, question={}, func={}".format(question, func.__name__))
 
         global current_cmd_info, root_cmd_info
 
@@ -491,6 +492,7 @@ class Dispatcher():
         self.cmd_words = None
         self.dispatch_cmd = None
         self.cmd_info = None
+        self.user_filters = self.config.data["user-filters"]
 
         global commands_by_name, current_dispatcher, dispatch_counter, commands, save_commands
 
@@ -657,7 +659,7 @@ class Dispatcher():
             value = []
     
             while tok != None:
-                if tok.startswith("--"):
+                if value and tok.startswith("--"):
                     break
                     
                 ev = self.expand_system_values(tok)
@@ -777,9 +779,9 @@ class Dispatcher():
 
         return tag_list, tok
 
-    def list_item_by_name(self, name, values):
+    def list_item_by_name(self, name, values, key="name"):
         # first, look for an exact match (takes priority)
-        matches = [value for value in values if name == value["name"]]
+        matches = [value for value in values if name == value[key]]
         if not matches:
             # look for abbreviated matches
             matches = [value for value in values if match(name, value["name"])]
@@ -1054,29 +1056,29 @@ class Dispatcher():
             single_with_comma = (tok == "," and tok2 is None)
             return single, single_with_comma
 
-    def add_quotes_to_string_args(self, args):
-        #console.print("BEFORE: self.args=", self.args)
+    # def add_quotes_to_string_args(self, args):
+    #     #console.print("BEFORE: self.args=", self.args)
 
-        for i, arg in enumerate(args):
+    #     for i, arg in enumerate(args):
 
-            # we currently process our options and those of ML app
+    #         # we currently process our options and those of ML app
 
-            #if arg.startswith("-"):
-            # parse option or tag: name=text
-            if "=" in arg:
-                # --option=value  (we only care about FIRST '=')
-                name, text = arg.split("=", 1)
-                single, single_with_comma = self.is_single_token(text)
+    #         #if arg.startswith("-"):
+    #         # parse option or tag: name=text
+    #         if "=" in arg:
+    #             # --option=value  (we only care about FIRST '=')
+    #             name, text = arg.split("=", 1)
+    #             single, single_with_comma = self.is_single_token(text)
 
-                if not single and not single_with_comma:
-                    # add back quotes that were string by the shell/command processor
-                    arg = name + '="' + text + '"'
-                    args[i] = arg
-            elif arg != "=":
-                # parse target or option value
-                if not self.is_single_token(arg):
-                    arg = '"' + arg + '"'
-                    args[i] = arg
+    #             if not single and not single_with_comma:
+    #                 # add back quotes that were string by the shell/command processor
+    #                 arg = name + '="' + text + '"'
+    #                 args[i] = arg
+    #         elif arg != "=":
+    #             # parse target or option value
+    #             if not self.is_single_token(arg):
+    #                 arg = '"' + arg + '"'
+    #                 args[i] = arg
 
 
     def validate_and_add_defaults(self, arguments, options, arg_dict):
@@ -1199,7 +1201,7 @@ class Dispatcher():
         self.dispatch_cmd = orig_text
 
         text = self.replace_curlies_with_quotes(orig_text)
-        console.diag("fixed cmd={}".format(text))
+        console.diag("\nADJUSTED cmd={}".format(text))
 
         scanner = Scanner(text)
         tok = scanner.scan()
@@ -1227,21 +1229,6 @@ class Dispatcher():
 
         self.cmd_info = cmd_info
 
-            # # user type incomplete command - display appropriate help
-            # if raise_syntax_exception:
-            #     errors.syntax_error("incomplete command")
-
-            # if command_help_func:
-            #     # parse any help-specific options
-            #     help_options = {}
-            #     self.parse_options(help_options, options, scanner, tok)
-
-            #     caller = self.impl_dict[command_help_func.__module__]
-            #     kwgroup_help_func(caller, cmd_info)
-            #     return
-            # else:
-            #     errors.env_error("no registered 'help' command")
-
         cmd_name = cmd_info["name"]
         self.cmd_words = cmd_name.replace("_", " ")
         func = cmd_info["func"]
@@ -1262,7 +1249,7 @@ class Dispatcher():
 
             caller = self.impl_dict[command_help_func.__module__]
             if self.preprocessor:
-                self.preprocessor(caller, arg_dict)
+                self.preprocessor(caller, arguments)
 
             command_help_func("help", caller, cmd_info)
             return
@@ -1330,29 +1317,101 @@ class Dispatcher():
         while tok and tok.startswith("--"):
             name = tok[2:]    # remove dashes
             match = self.list_item_by_name(name, options)
+            if match:
+                tok = self.parse_cmd_option(match, scanner, tok, arg_dict)
+            else:
+                if self.cmd_info["user_filters"]:
+                    # see if option name is the name of a user-defined filter
+                    match_name = self.list_item_by_value(name, self.user_filters.keys())
+                    if match_name:
+                        match = self.user_filters[match_name]
+                        tok = self.parse_user_filter(match, scanner, tok, arg_dict)
+
             if not match:
                 self.syntax_error("cmd '{}' doesn't have an option named: {}".format(self.cmd_words, tok))
+ 
+        return tok
 
-            name = match["name"]        # full name
-            tok = scanner.scan()        # skip over name
-            value = True
-            found_equals = False
+    def parse_user_filter(self, match, scanner, tok, arg_dict):            
 
-            if tok == "=":
-                found_equals = True
-                tok = scanner.scan()        # skip over equals
+        tok = scanner.scan()        # skip over name
+        
+        match_prop = match["prop"]
+        match_type = match["type"]
+        match_op = utils.safe_value(match, "op")
+        match_value = utils.safe_value(match, "value")
 
-            values = match["values"] if "values" in match else None
-            #value = self.process_option_value(name, match["type"], value, values)
-            value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner)
+        if match_op and match_value:
+            # user filter is a complete filter
+            filter = {"prop": match_prop, "op": match_op, "value": match_value}
+        else:
+            filter = {"prop": match_prop}
 
-            arg_dict[name] = value
+            ops = ["=", "<", "<=", ">", ">=", "==", "!=", "<>"]
+            if tok in ["=", "<", "<=", ">", ">=", "==", "!=", "<>"]:
+                compare_op = tok
+                tok = scanner.scan()        # skip over comparison op
+            else:
+                compare_op = "="
+
+            filter["op"] = compare_op
+
+            values = None
+            required = False
+
+            value, tok = self.parse_option_value(match_prop, match_type, values, compare_op, tok, scanner, 
+                required)
+
+            filter["value"] = value
+
+        # add filter to filter list in arg_dict
+        if not "filter" in arg_dict:
+            arg_dict["filter"] = []
+
+        arg_dict["filter"].append(filter)
+        return tok
+
+    def parse_cmd_option(self, match, scanner, tok, arg_dict):            
+
+        tok = scanner.scan()        # skip over name
+        value = True
+        found_equals = False
+        name = match["name"]        # full name
+
+        if tok == "=":
+            found_equals = True
+            tok = scanner.scan()        # skip over equals
+
+        values = match["values"] if "values" in match else None
+        #value = self.process_option_value(name, match["type"], value, values)
+        required = utils.safe_value(match, "required")
+
+        value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner, 
+            required)
+
+        arg_dict[name] = value
 
         return tok
 
-    def parse_option_value(self, name, opt_type, keywords, found_equals, tok, scanner):
+    def parse_option_value(self, name, opt_type, keywords, found_equals, tok, scanner, required=False):
 
-        if opt_type == "str_list":
+        if keywords:
+            if required is None:
+                # when keywords are specified, the default is that the value is required
+                required = True
+
+            user_value = self.expand_system_values(tok)
+
+            tok = scanner.scan()
+            value = self.match_keyword(user_value, keywords)
+            if value is None:
+                if required:
+                    self.syntax_error("value specified for option --{} ({}) is not one of required values: {}".\
+                        format(name, user_value, ", ".join(keywords)))
+                else:
+                    value = user_value            
+
+        elif opt_type == "str_list":
             value, tok = self.parse_string_list(tok, scanner)
             if len(value)==0 and required:
                 self.syntax_error("missing value for required option: " + name)
@@ -1390,10 +1449,6 @@ class Dispatcher():
                 value = self.parse_flag_value(name, value)
             else:
                 value = True
-        elif keywords:
-            value = tok
-            tok = scanner.scan()
-            value = self.match_keyword(value, keywords)
         else:
             # its a simple value
             value = tok
