@@ -220,22 +220,6 @@ class Run():
         # local_output_dir where we will use XT mirroring to push their changes to "mirroring" path 
         # in run's storage
 
-        # # debug blobfuse path problem
-        # fn = os.getenv("XT_OUTPUT_DIR") + "/test.txt"
-        # print("TESTING ability to write new file:", fn)
-        # file_utils.ensure_dir_exists(file=fn)
-        # with open(fn, "wt") as outfile:
-        #     outfile.write("testing...\n")
-        # print("test PASSED", flush=True)
-
-        # # debug blobfuse path problem
-        # fn =  os.getenv("XT_NODE_DIR") + "/test.txt"
-        # print("TESTING ability to write new file:", fn)
-        # file_utils.ensure_dir_exists(file=fn)
-        # with open(fn, "wt") as outfile:
-        #     outfile.write("testing...\n")
-        # print("test PASSED", flush=True)
-
         serial_num = random.randint(1,100000)
         log_dir = "{}/logs/{}".format(self.tensorboard_path, serial_num)
 
@@ -331,7 +315,7 @@ class Run():
             log_records = []
 
             self.store.end_run(self.ws_name, self.run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-                aggregate_dest=None, dest_name=None, is_aml=True)
+                aggregate_dest=None, dest_name=None, is_aml=True, job_id=self.job_id)
 
             self.store.update_run_at_end(self.ws_name, self.run_name, status, exit_code, end_time, log_records, 
                 hparams_dict, metrics_rollup_dict)
@@ -352,7 +336,7 @@ class Run():
         #console.print("log_hparam, self.store=", self.store)
 
         if self.store and self.xt_logging:
-            self.store.log_run_event(self.ws_name, self.run_name, "hparams", data_dict)
+            self.store.log_run_event(self.ws_name, self.run_name, "hparams", data_dict, job_id=self.job_id)
 
         if self.is_aml and self.aml_logging:
             self.aml_run.log_row("hparams", **data_dict)
@@ -392,7 +376,7 @@ class Run():
 
         if self.store and self.xt_logging:
             #console.print("logging run_event for metrics...")
-            self.store.log_run_event(self.ws_name, self.run_name, "metrics", dd)
+            self.store.log_run_event(self.ws_name, self.run_name, "metrics", dd, job_id=self.job_id)
 
             if metric_names_changed:
                 ddx = {"metric_names": list(self.metric_names)}
@@ -482,17 +466,37 @@ class Run():
 
         return dict_cp
 
+    def delete_run_subfolder_in_store(self, run_subfolder, show_feedback=False):
+
+        if self.store:
+            #store_path = "$jobs/{}/runs/run/output".format(self.job_id, self.run_name)
+            store_path = "$" + store_utils.get_run_path(self.job_id, self.run_name) + "/" + run_subfolder
+            count = self.store.delete_run_subfolder_and_files(self.ws_name, self.run_name, store_path)
+        
+    def upload_run_subfolder_to_store(self, local_path=None, run_subfolder="output", show_feedback=False):
+
+        # # TEMP TEMP TEMP
+        # self.job_id = "job1837"
+        # self.run_name = "run6146"
+        # self.ws_name = "tpx"
+
+        if self.store:
+
+            if not local_path:
+                local_path = os.getenv("XT_OUTPUT_DIR", "output")
+
+            # bug workaround: force self.store usage to avoid problem calling store_utils
+            result = self.store.does_job_exist(self.ws_name, self.job_id)
+            
+            #store_path = "$jobs/{}/runs/run/output".format(self.job_id, self.run_name)
+            store_path = "$" + store_utils.get_run_path(self.job_id, self.run_name) + "/" + run_subfolder
+            storage = impl_storage.ImplStorage(self.config, self.store)
+
+            count = storage.impl_storage_api.upload(local_path=local_path, store_path=store_path, share=None, feedback=show_feedback, 
+                workspace=self.ws_name, run=None, experiment=None, job=None, show_output=show_feedback)
+    
     def upload_output_dir(self, local_path=None, show_feedback=False):
-
-        if not local_path:
-            local_path = os.getenv("XT_OUTPUT_DIR", "output")
-
-        #store_path = "$jobs/{}/runs/run/output".format(self.job_id, self.run_name)
-        store_path = "$" + store_utils.get_run_path(self.job_id, self.run_name) + "/output"
-        storage = impl_storage.ImplStorage(self.config, self.store)
-
-        count = storage.upload(local_path=local_path, store_path=store_path, share=None, feedback=show_feedback, 
-            workspace=self.ws_name, run=None, experiment=None, job=None)
+        return self.upload_run_subfolder_to_store(local_path, "output")
 
     def upload_files_to_share(self, share, store_path, local_path, show_feedback=False):
         '''
