@@ -1044,7 +1044,7 @@ def wrapup_runs_for_nodes(store, ws_name, job_id, nodes, max_workers):
                 context_plus = context_by_nodes[node_id]
                 context = context_plus["runs"][0]
 
-                wrapup_run_with_context(store, run, context)
+                wrapup_run_with_context(store, run, context, job_id=job_id)
 
                 with worker_lock:
                     nonlocal next_progress_num
@@ -1062,8 +1062,11 @@ def wrapup_runs_for_nodes(store, ws_name, job_id, nodes, max_workers):
 
     fb.feedback("done", is_final=True)
 
-def wrapup_run_with_context(store, run, context_dict):
+def wrapup_run_with_context(store, run, context_dict, job_id=None):
     context = utils.dict_to_object(context_dict)
+    if job_id:
+        context.job_id = job_id
+        
     status = "cancelled"
     exit_code = 0
     node_id = utils.node_id(context.node_index)
