@@ -13,7 +13,7 @@ import psutil
 import logging
 from threading import Lock
 
-from xtlib import utils
+from xtlib import mirror_worker, utils
 from xtlib import errors
 from xtlib import file_utils
 from xtlib.utils import log_info, log_title
@@ -167,6 +167,7 @@ class RunInfo():
         '''wrap-up the run (logging, capture)'''
         #console.print("run_wrapup, self.killed_for_restart={}".format(self.killed_for_restart))
         
+        log_info("run_wrapup: mirror_worker", mirror_worker)
         if self.mirror_worker:
             self.mirror_close_func(self)
 
