@@ -996,7 +996,7 @@ class Runner():
             self.write_node_tags_data(id_for_node, ws_name, job_id, td)
 
         # log QUEUED event for flat/parent run        
-        self.store.log_run_event(workspace, run_name, "status-change", {"status": "queued"}) 
+        self.store.log_run_event(workspace, run_name, "status-change", {"status": "queued"}, job_id=job_id) 
 
     def log_runs_queued(self, runs_by_box, ws_name, job_id, total_run_count, secrets_by_node, 
         service_info_by_node, args):
@@ -1285,6 +1285,16 @@ class Runner():
 
         return (cmds, runsets) if cmds else None
 
+    def gen_runsets_report(self, runsets):
+        fn = os.path.expanduser("~/.xt/runsets.txt")
+        fn = os.path.abspath(fn)
+
+        with open(fn, "wt") as outfile:
+            for rs in runsets:
+                outfile.write("runset: {}\n".format(rs))
+
+        console.print("--> to view generated HP runsets, see the file: {}".format(fn))
+
     def build_cmds_with_search(self, service_type, cmd_parts, parent_script, run_script, run_cmd_from_script, 
         use_controller, dry_run, args):
         '''
@@ -1336,6 +1346,12 @@ class Runner():
 
         # HPARAM SEARCH
         cmds, runsets, sweeps_text, new_run_cmd = self.build_static_hparam_cmds(run_cmd, node_count, args)
+        
+        # NOTE: if runsets is not None, then this is a static search where user needs YAML files generated for each child run 
+        #       if this is a static search and runsets is None, then runsets have already been applied to cmdlines
+
+        if runsets is not None:
+            show_run_report = False
             
         using_hp = not(not sweeps_text)
         if using_hp and use_aml_for_hparam:
@@ -1343,11 +1359,14 @@ class Runner():
             # for AML hyperdrive, we pass only constant args from cmd_parts
             #cmd_parts = [tp for tp in template_parts if tp != '{}']
 
+        if runsets:
+            self.gen_runsets_report(runsets)
+
         if cmds:
             # STATIC HPARAM SEARCH
             #run_count = len(runsets) if runsets else len(cmds)
             run_count = len(cmds)
-            runsets = None      # do not process further in controller
+            #runsets = None      # do not process further in controller
             search_style = "static"
 
         runs = args["runs"]
@@ -1499,9 +1518,11 @@ class Runner():
                         # apply runsets to run_cmd to generate cmd list
                         cmd_line_base = run_cmd
                         run_cmds = hp_client.generate_runs(runsets, cmd_line_base)
+                        runsets = None    # since we have applied them to run_cmds
                     else:
-                        # will use runsets at pre-run time to generate a runset yaml file
-                        run_cmds = [cmd_line]
+                        # controller will use runsets when launching each child run (and generate a runset yaml file to be read by that child)
+                        # run_cmds = [cmd_line]
+                        run_cmds = len(runsets)*[cmd_line]   # duplicate the cmd_line for each run_set
 
                 else:
                     # dynamic HP
