@@ -168,7 +168,7 @@ def add_controller_env_vars(env_vars, config, box_secret, node_id):
         - XT_SERVER_CERT            (controller only, for rpyc socket)
         - XT_USER_PRINCIPLE_NAME    (controller only, for request validation)
         - XT_STORE_CREDS            (controller (store access for remote xt) + ML app)
-        - XT_MONGO_CONN_STR         (controller (store access for remote xt) + ML app)
+        - XT_DB_CREDS               (controller (store access for remote xt) + ML app)
         - XT_STORE_CODE_PATH        (controller (store access for remote xt) + ML app)
         - XT_NODE_ID                (controller (index into multirun file) + ML app)
     '''
@@ -185,17 +185,18 @@ def add_controller_env_vars(env_vars, config, box_secret, node_id):
     sc = json.dumps(storage_creds)
     env_vars["XT_STORE_CREDS"] = utils.text_to_base64(sc)
 
-    mongo_creds, mongo_name = config.get_mongo_creds()
+    mongo_creds = config.get_database_creds()
 
     ss_info = config.get_store_info()
     storage_tracking = utils.safe_value(ss_info, "track-storage")
 
     if storage_tracking:
         # no mongo is available for run to use (everything is logged to storage)
-        env_vars["XT_MONGO_CONN_STR"] = ""
+        env_vars["XT_DB_CREDS"] = ""
     else:
-        mongo_conn_str = mongo_creds["mongo-connection-string"]
-        env_vars["XT_MONGO_CONN_STR"] = utils.text_to_base64(mongo_conn_str)  
+        db_creds = config.get_database_creds()
+        db_creds_json = json.dumps(db_creds)
+        env_vars["XT_DB_CREDS"] = utils.text_to_base64(db_creds_json)
 
     # NORMAL strings
     #env_vars["XT_USER_PRINCIPLE_NAME"] = config.get_vault_key("user_principle_name")
