@@ -3,6 +3,8 @@
 # Licensed under the MIT license.
 #
 # mongo_v2.py: new XT API for talking to mongo about RUN and JOB data
+
+# NOTE: storage v2 is not currently supported (quicktest is broken; implementation is incomplete)
 import os
 from re import sub
 import uuid
@@ -791,7 +793,7 @@ class MongoDB2(implements(DbInterface)):
             dd["_id"] = _id
             dd["ws_name"] = ws_name
             update_doc = { "$set": dd}
-            self.mongo_with_retries("update_run_info", \
+            self.mongo_with_retries("update_run_info", dbx, \
                 lambda: dbx["run_info"].update_one( {"_id": _id}, update_doc, upsert=True))
 
         # update RUN_STATS
@@ -1798,15 +1800,21 @@ class MongoDB2(implements(DbInterface)):
         needed["service_info_by_node"] = default
         needed["job_info"] = default
         needed["hparams"] = default
+        needed["tags"] = default
 
         if fields_dict:
             orig_fd = dict(fields_dict)
 
             for name, value in orig_fd.items():
-                if name.startswith("job_stats") or name.startswith("tags.") or name in job_helper.job_stats_props:
+
+                if name.startswith("job_stats") or name in job_helper.job_stats_props:
                     needed["job_stats"] = value
                     del fields_dict[name]
 
+                elif name == "tags" or name.startswith("tags."):
+                    needed["tags"] = value
+                    del fields_dict[name]
+
                 elif name.startswith("connect_info"):
                     needed["connect_info"] = value
                     del fields_dict[name]
@@ -1895,6 +1903,13 @@ class MongoDB2(implements(DbInterface)):
             self.join_data(records, hparams, embed_name="hparams")
             self.round_trip_count += 1
 
+        # add TAGS
+        if needed["tags"]:
+            # NOTE: storage v2 is not currently supported (quicktest is broken; implementation is incomplete)
+            tags = self.get_collection(ws_name, "job_tags", {"_id": {"$in": ids}})
+            self.join_data(records, tags, embed_name="tags")
+            self.round_trip_count += 1
+
     def join_data(self, records, infos, info_key="_id", embed_name=None):
         # build a dict to quickly find records
         rd = {r["_id"]: r for r in records}
