@@ -61,7 +61,7 @@ def store_from_context(context):
     db_creds = context.db_creds
 
     store = Store(context.store_creds, provider_code_path=context.store_code_path, 
-        db_creds=db_creds, db_options=db_options)
+        db_creds=db_creds, db_options=db_options, show_missing_jobid_calls=context.show_missing_jobid_calls)
 
     return store
 
@@ -95,7 +95,7 @@ def is_version1_store(config=None, storage_creds=None, provider_code_path=None):
 
 # public function
 def Store(storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  db_creds=None, 
-        provider_code_path=None, validate_creds=False, db_options=None):
+        provider_code_path=None, validate_creds=False, db_options=None, show_missing_jobid_calls=False):
     '''
     This function peeks at the version number in the storage and chooses StoreV1 or StoreV2 as the store 
     class to return an instance of.
@@ -125,7 +125,7 @@ def Store(storage_creds=None, config=None, max_retries=25, run_cache_dir=None, f
         assert not provider.does_container_exist(constants.INFO_CONTAINER_V1)
 
         store = StoreV2(storage_creds, config, max_retries, run_cache_dir, feedback_enabled,  db_creds, 
-            provider_code_path, validate_creds, db_options=db_options)
+            provider_code_path, validate_creds, db_options=db_options, show_missing_jobid_calls=show_missing_jobid_calls)
 
         assert not provider.does_container_exist(constants.INFO_CONTAINER_V1)
 
@@ -155,10 +155,11 @@ class StoreV2():
     '''
     
     def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  db_creds=None, 
-        provider_code_path=None, validate_creds=False, db_options=None):
+        provider_code_path=None, validate_creds=False, db_options=None, show_missing_jobid_calls=False):
         '''This is the constructor for the Store class. '''
 
         self.feedback_enabled = feedback_enabled
+        self.show_missing_jobid_calls = show_missing_jobid_calls
 
         if not db_creds and config:
             db_creds = config.get_database_creds()
@@ -625,7 +626,7 @@ class StoreV2():
             sku=sku, search_style=search_style, is_child=True, run_index=run_index)           
 
         # finally, add a "child_created" record to the parent
-        self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name})
+        self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, job_id=job_id)
 
     def start_run_core(self, ws_name, run_name, exper_name=None, description=None, username=None,
             box_name=None, app_name=None, repeat=None, is_parent=False, job_id=None, pool=None, node_index=None,
@@ -727,7 +728,7 @@ class StoreV2():
         return result
 
     def end_run(self, ws_name, run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-         aggregate_dest=None, dest_name=None):
+         aggregate_dest=None, dest_name=None, job_id=None):
 
         utils.log_info("end_run", run_name)
 
@@ -735,7 +736,7 @@ class StoreV2():
             end_time = utils.get_arrow_time_str()
 
         self.log_run_event(ws_name, run_name, "ended", {"status": status, "exit_code": exit_code, \
-            "metrics_rollup": metrics_rollup_dict}, event_time=end_time)
+            "metrics_rollup": metrics_rollup_dict}, event_time=end_time, job_id=job_id)
 
         # append "end" record to workspace summary log
         end_record = {"ws_name": ws_name, "run_name": run_name, "time": end_time, "event": "end", 
@@ -744,7 +745,7 @@ class StoreV2():
         text = json.dumps(end_record) + "\n"
 
         # append to summary file in run dir
-        self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
+        self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text, job_id=job_id)
 
     def delete_run(self, ws_name, run_name):
         return self.helper.delete_run(ws_name, run_name)
@@ -812,7 +813,7 @@ class StoreV2():
         return nodes_read
             
     def rollup_and_end_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
-        report_rollup, use_last_end_time=False, node_id=None):
+        report_rollup, use_last_end_time=False, node_id=None, job_id=None):
 
         utils.log_info("rollup_and_end_run", run_name)
 
@@ -845,7 +846,7 @@ class StoreV2():
         utils.log_info("rolled up metrics", metrics)
 
         self.end_run(ws_name, run_name, status, exit_code, hparams, metrics,
-            end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name)
+            end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name, job_id=job_id)
 
         # this causes needless db traffic, so don't do it
         # if self.database:
@@ -968,7 +969,7 @@ class StoreV2():
             # LOG "ENDED" to run_log, APPEND TO ALLRUNS
             self.rollup_and_end_run(ws_name, run_name, aggregate_dest, dest_name, status, exit_code, 
                 primary_metric=primary_metric, maximize_metric=maximize_metric, report_rollup=report_rollup, 
-                node_id=node_id)
+                node_id=node_id, job_id=job_id)
 
         if rundir and capture_files:
             # CAPTURE OUTPUT FILES
@@ -994,7 +995,8 @@ class StoreV2():
                     log_info("upload exception", ex) 
 
             elapsed = time.time() - started
-            self.log_run_event(ws_name, run_name, "capture_after", {"elapsed": elapsed, "count": len(copied_files)})
+            self.log_run_event(ws_name, run_name, "capture_after", {"elapsed": elapsed, "count": len(copied_files)}, 
+                job_id=job_id)
 
         # tell database RUNS that this run has completed
         if self.database:
@@ -1036,7 +1038,6 @@ class StoreV2():
     def append_run_file(self, ws_name, run_name, run_fn, text, job_id=None):
         '''append 'text' to the run file 'run_fn'.
         '''
-        #return self.helper.append_run_file(ws_name, run_name, run_fn, text)
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.append_file(run_fn, text)
 
@@ -1047,18 +1048,18 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.read_file(run_fn)
 
-    def upload_file_to_run(self, ws_name, run_name, run_fn, source_fn):
+    def upload_file_to_run(self, ws_name, run_name, run_fn, source_fn, job_id=None):
         '''upload the local file 'source_fn' as the run file 'run_fn'.
         '''
         #return self.helper.upload_file_to_run(ws_name, run_name, run_fn, source_fn)
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.upload_file(run_fn, source_fn)
 
     def upload_files_to_run(self, ws_name, run_name, run_folder, source_wildcard, exclude_dirs_and_files=[]):
         '''upload the local files specified by 'source_wildcard' to the run folder 'run_folder'.
         '''
         #return self.helper.upload_files_to_run(ws_name, run_name, run_folder, source_wildcard, exclude_dirs)
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=self.job_id)
         return rf.upload_files(run_folder, source_wildcard, exclude_dirs_and_files=exclude_dirs_and_files)
 
     def read_log_records(self, ws_name, run_name):
@@ -1105,15 +1106,14 @@ class StoreV2():
         '''download the run file 'run_fn' to the local file 'dest_fn'.
         '''
         dest_fn = os.path.abspath(dest_fn)      # ensure it has a directory specified
-        #return self.helper.download_file_from_run(ws_name, run_name, run_fn, dest_fn)
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.download_file(run_fn, dest_fn)
 
-    def download_files_from_run(self, ws_name, run_name, run_wildcard, dest_folder):
+    def download_files_from_run(self, ws_name, run_name, run_wildcard, dest_folder, job_id=None):
         '''download the run files specified by 'run_wildcard' to the local folder 'dest_folder'.
         '''
         #return self.helper.download_files_from_run(ws_name, run_name, run_wildcard, dest_folder)
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.download_files(run_wildcard, dest_folder)
 
     def get_run_filenames(self, ws_name, run_name, run_wildcard=None, full_paths=False):
@@ -1129,6 +1129,12 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.delete_file(filename)
 
+    def delete_run_subfolder_and_files(self, ws_name, run_name, subfolder_name):
+        '''delete all files under the specified folder_name.
+        '''
+        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        return rf.delete_files(subfolder_name + "/**")
+
     def does_run_file_exist(self, ws_name, run_name, run_fn):
         '''return True if the specified run file 'run_fn' exists.
         '''
@@ -1311,6 +1317,14 @@ class StoreV2():
         return self.helper.workspace_files(ws_name, use_blobs)
 
     def run_files(self, ws_name, run_name, use_blobs=False, job_id=None):
+        if job_id is None and self.show_missing_jobid_calls:
+
+            console.print("internal warning: job_id is None in call to run_files() [will run slower as a result]")
+            print("----> diagnostic stack trace")
+            import traceback
+            traceback.print_stack()
+            print("<------")
+
         return self.helper.run_files(ws_name, run_name, use_blobs, job_id=job_id)
 
     def experiment_files(self, ws_name, exper_name, use_blobs=False):
