@@ -18,6 +18,7 @@ from azure.storage.blob import AppendBlobService, BlockBlobService, PublicAccess
 from xtlib import utils
 from xtlib import errors
 from xtlib import console
+from xtlib import file_utils
 from xtlib.storage.store_interface import StoreInterface
 
 logger = logging.getLogger(__name__)
@@ -319,7 +320,7 @@ class AzureBlobStore21(implements(StoreInterface)):
         self._container_check(container, create_if_needed=False)
 
         # ensure path has correct slashes 
-        dest_fn = os.path.relpath(dest_fn)
+        dest_fn = os.path.abspath(dest_fn)
         dest_dir = os.path.dirname(dest_fn)
         if not dest_dir:
             dest_dir = "."
