@@ -653,9 +653,9 @@ class StoreBlobObjs():
         self._check_ws_name(ws_name)
 
         if self.does_workspace_exist(ws_name):
-            #errors.store_error("workspace already exists: {}".format(ws_name))
+            errors.store_error("workspace already exists: {}".format(ws_name))
             # TEMP TEMP TEMP
-            return
+            #return
 
         # note: this operation often must retry several times if same container has just been deleted
         #console.print("creating workspace=", ws_name)
