@@ -1009,4 +1009,9 @@ class MongoDB(implements(DbInterface)):
         # not supported in mongo_v1
         pass
 
-            
\ No newline at end of file
+    # API call
+    def set_job_status(self, ws_name, job_id, status):
+
+        if self.update_job_stats:
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$set": {"job_status": status} })
+            self.mongo_with_retries("set_job_status", cmd)
