@@ -684,7 +684,7 @@ class StoreV1():
         return text, last_end_time
 
     def rollup_and_end_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
-        report_rollup, use_last_end_time=False):
+        report_rollup, use_last_end_time=False, job_id=None):
 
         # if use_last_end_time:
         #     end_time = last_end_time
@@ -930,7 +930,7 @@ class StoreV1():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.read_file(run_fn)
 
-    def upload_file_to_run(self, ws_name, run_name, run_fn, source_fn):
+    def upload_file_to_run(self, ws_name, run_name, run_fn, source_fn, job_id=None):
         '''upload the local file 'source_fn' as the run file 'run_fn'.
         '''
         #return self.helper.upload_file_to_run(ws_name, run_name, run_fn, source_fn)
@@ -948,11 +948,11 @@ class StoreV1():
         '''download the run file 'run_fn' to the local file 'dest_fn'.
         '''
         dest_fn = os.path.abspath(dest_fn)      # ensure it has a directory specified
-        #return self.helper.download_file_from_run(ws_name, run_name, run_fn, dest_fn)
+        print("download_file_from_run: dest_fn=", dest_fn)
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.download_file(run_fn, dest_fn)
 
-    def download_files_from_run(self, ws_name, run_name, run_wildcard, dest_folder):
+    def download_files_from_run(self, ws_name, run_name, run_wildcard, dest_folder, job_id=None):
         '''download the run files specified by 'run_wildcard' to the local folder 'dest_folder'.
         '''
         #return self.helper.download_files_from_run(ws_name, run_name, run_wildcard, dest_folder)
@@ -972,6 +972,12 @@ class StoreV1():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.delete_file(filename)
 
+    def delete_run_subfolder_and_files(self, ws_name, run_name, subfolder_name):
+        '''delete all files under the specified folder_name.
+        '''
+        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        return rf.delete_files(subfolder_name + "/**")
+
     def does_run_file_exist(self, ws_name, run_name, run_fn):
         '''return True if the specified run file 'run_fn' exists.
         '''
