@@ -11,19 +11,24 @@ REPLACE ALL OF THE BELOW with their version 1 or version 2 equivalents
 from xtlib import constants
 
 def make_share_name(name):
-   raise Exception("store-utils not yet initialized")
+   #raise Exception("store-utils not yet initialized")
+   pass
 
 def is_share_name(name):
-   raise Exception("store-utils not yet initialized")
+   #raise Exception("store-utils not yet initialized")
+   pass
 
 def get_run_path(job_id, run_name):
-   raise Exception("store-utils not yet initialized")
+   #raise Exception("store-utils not yet initialized")
+   pass
 
 def get_jobs_container(ws_name):
-   raise Exception("store-utils not yet initialized")
+   #raise Exception("store-utils not yet initialized")
+   pass
 
 def make_id(ws_name, obj_name):
-   raise Exception("store-utils not yet initialized")
+   #raise Exception("store-utils not yet initialized")
+   pass
 
 # version independent function
 def simplify_id(record):
