@@ -4,11 +4,14 @@
 #
  # py: common functions shared among XT modules
 
+from logging import error
 import os
 import re
 import sys
 import json
+import math
 import time
+from xtlib import pc_utils
 import arrow
 import pickle
 import logging
@@ -16,11 +19,14 @@ import shutil
 import datetime
 import traceback
 import importlib
+from threading import Thread, Lock
+from multiprocessing import Pool
 
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import job_helper
+from xtlib import run_errors
 
 from xtlib.console import console
 from xtlib.helpers.feedbackParts import feedback as fb
@@ -92,6 +98,23 @@ def format_workspace_exper_run(store_type, ws_name, exper_name, run_name):
     return "{}/{}.format(ws_name, run_name)"
 
 def time_diff(time1, time2):
+    '''
+    Args:
+        time1, time2: datetime or arrow objects 
+    '''
+    atype = arrow.arrow.Arrow
+    dttype = datetime.datetime
+
+    # support mixed types
+    if type(time1) != type(time2):
+
+        # convert both to datetime without timezone
+        if isinstance(time1, atype):
+            time1 = time1.datetime.replace(tzinfo=None)
+
+        if isinstance(time2, atype):
+            time2 = time2.datetime.replace(tzinfo=None)
+
     return (time1 - time2).total_seconds()
 
 def elapsed_time(start):
@@ -144,7 +167,7 @@ def format_elapsed_hms(elapsed, include_fraction=False):
 
     return value
 
-def make_retry_func(max_retries=8):
+def make_retry_func(max_retries=8, reset_func=None):
     #max_retries = 8     # 95 secs total retry time
     #console.print("received max_retries=", max_retries)
 
@@ -160,11 +183,27 @@ def make_retry_func(max_retries=8):
         if context.response and context.response.status:
             status = context.response.status
 
+        ex_type, exception_msg, ex_traceback = sys.exc_info()
+        str_exception = str(context.exception).replace('\n', '')
+
+        '''
+        do not retry this error; it must be retried at the API level:
+            azure error being RETRIED (non-fatal): status=412, retry count=21, backoff=16, max_retries=25, exception=The condition 
+            specified using HTTP conditional header(s) is not met. ErrorCode: ConditionNotMet<?xml version="1.0" encoding="utf-8"?>
+        '''
+
+        if status == 412 or "specified using HTTP conditional header(s) is not met" in str_exception:
+            # do not retry - fail now
+            return None
+
+        tb_lines = get_stack_lines()
+        run_errors.record_run_error("storage", str_exception, None, tb_lines)
+
         with open(constants.AZURE_ERRORS_FN, "a") as errfile:
             error_time = time.time()
-            exception_msg = sys.exc_info()[1]
 
-            console.print("Azure Exception in XTLIB (see {} at time={} for more details): {}".format(constants.AZURE_ERRORS_FN, error_time, exception_msg))
+            console.print("Azure Exception in XTLIB (see {} at time={} for more details): {}".\
+                format(constants.AZURE_ERRORS_FN, error_time, exception_msg))
 
             # console.print exception and stack trace in errfile
             errfile.write("Azure Exception in XTLIB at time={}: {}\n".format(error_time, exception_msg))
@@ -178,7 +217,6 @@ def make_retry_func(max_retries=8):
                 console.print("*** auzre error retry FAILED (FATAL): max_retries={} exceeded ***".format(max_retries))
         else:
             backoff_time = min(16, 2**context.count)
-            str_exception = str(context.exception).replace('\n', '')
 
             if max_retries:
                 console.print("\nazure error being RETRIED (non-fatal): status={}, retry count={}, backoff={}, max_retries={}, exception={}" \
@@ -279,11 +317,17 @@ def get_python_value_from_text(part):
 
     return value        
 
-def get_time(arrow_time=None):
+def get_arrow_time_str():
+    ''' 
+    returns NOW as an arrow time string
+    '''
     #return time.time()
     return str(arrow.now()) 
 
 def get_time_from_arrow_str(arrow_str):
+    '''
+    converts an arrow time string to seconds (float)
+    '''
     arrow_time = arrow.get(arrow_str)
     return arrow_time.datetime.timestamp()
 
@@ -312,9 +356,8 @@ def safe_value(dd, key, default=None):
 
 def safe_delete(dd, key, default=None):
     value = default
-    if key in dd:
-        value = dd[key]
-        del dd[key]
+    if dd and key in dd:
+        value = dd.pop(key)
 
     return value
 
@@ -325,7 +368,8 @@ def safe_move(newdd, dd, key, default=None, flatten=False):
         del dd[key]
 
         if flatten:
-            newdd.update(value)
+            if value:
+                newdd.update(value)
         else:
             newdd[key] = value
 
@@ -548,3 +592,162 @@ def report_elapsed(start, name):
     elapsed = time.time() - start
     print("{} took: {:.2f} secs".format(name, elapsed))
 
+def run_on_threads(worker_func, workitems, max_workers, args_for_worker):
+    # wrapup a subset of run_list on each worker thread
+    if not workitems:
+        # no work to do
+        return
+
+    threads = []
+    item_count = len(workitems)
+
+    #for runs in runs_by_box.values():
+    num_workers = min(item_count, max_workers)
+
+    # split id_list into num_workers sub-lists
+    per_list = math.ceil(item_count/num_workers)
+    next_index = 0
+
+    while next_index < item_count:
+        # build next sub-list
+        items = workitems[next_index:next_index+per_list]
+        full_args = [items] + args_for_worker
+        next_index += per_list
+
+        thread = Thread(target=worker_func, args=full_args)
+        thread.start()
+        threads.append(thread)
+
+    # wait for all threads to complete their work
+    for thread in threads:
+        thread.join()
+
+def run_on_processes(worker_func, workitems, max_workers, args_for_worker):
+    # wrapup a subset of run_list on each worker process
+    item_count = len(workitems)
+
+    #for runs in runs_by_box.values():
+    num_workers = min(item_count, max_workers)
+
+    # split id_list into num_workers sub-lists
+    per_list = math.ceil(item_count/num_workers)
+    next_index = 0
+
+    pool = Pool(processes=num_workers) 
+
+    res = [pool.apply_async(worker_func, args=(item,)) for item in workitems]
+
+    # wait for all processes to complete
+    results = [p.get() for p in res]
+
+def log_title(title, double=False):
+    if pc_utils.is_compute_node():
+        if double:
+            line = "=========================================="
+        else:
+            line = "------------------------------------------"
+
+        console.print(line)
+        console.print(title)
+        console.print(line)
+
+add_time_to_log_info = False
+
+def log_info(title, value):
+    if pc_utils.is_compute_node():
+        title = (title + ":").ljust(25)
+
+        # add time for platforms that don't log time for console output
+        if add_time_to_log_info:
+            now = datetime.datetime.now()
+            now_str = now.strftime("%Y-%m-%d %H:%M:%S")
+            title = "@{}: {}".format(now_str, title)
+
+        console.print("{} \t{}".format(title, value))
+
+def log_info_to_text(title, value):
+    title = (title + ":").ljust(25)
+    text = "{} \t{}".format(title, value)
+    return text
+
+def load_context_from_mrc():
+    context = None
+
+    fn_context = os.path.abspath(constants.FN_RUN_CONTEXT)
+    if os.path.exists(fn_context):
+        json_context = file_utils.read_text_file(fn_context)
+
+        context_dict = json.loads(json_context)
+        context = dict_to_object(context_dict)
+
+    return context
+    
+def get_stack_lines():
+    ''' 
+    Processing:
+        - this returns the formatted lines of the CALL STACK, from current frame to beginning of program.
+    
+    NOTE: this does NOT return TRACEBACK frames, which are the frames between the current stack frame and 
+    the frame that threw an exception.
+    '''
+
+    RUNPY = "lib\\runpy.py" if os.name == "nt" else "lib/runpy.py"
+    frames = traceback.extract_stack()
+
+    index = len(frames)-1
+    while index > 0:
+        if frames[index].filename.endswith(RUNPY):
+            # remove earlier frames
+            frames = frames[index:]
+            break
+        index -= 1
+
+    # remove 2 most recent frames (part of our tooling)
+    frames.pop()
+    frames.pop()
+
+    lines = []
+    for frame in frames:
+        line = 'File "{}", line {}, in {}'.format(frame[0], frame[1], frame[2])
+        lines.append(line)
+        line = '{}'.format(frame[3])
+        lines.append(line)
+    
+    ex_type, ex_value, ex_traceback = sys.exc_info()
+    last_line = "{}: {}".format(ex_type, ex_value)
+    lines.append(last_line)
+
+    return lines
+
+def rename_dict_key(dd, from_name, to_name):
+    if from_name in dd and from_name != to_name:
+        dd[to_name] = dd.pop(from_name)
+
+def is_col_name(text):
+    # must start with letter, followed by letter/digit/_/-/.
+    is_name = False
+    if text:
+        is_name = re.search("[a-z, A-Z][\\w\\-\\.]*", text)
+
+    return is_name
+
+def print_env_vars():
+    keys = list(os.environ.keys())
+    keys.sort()
+    for key in keys:
+        value = os.getenv(key)
+        log_info(key, value)
+    
+def tag_dict_from_list(tag_list):
+    td = {}
+
+    if tag_list:
+        # convert to dict
+        for tag in tag_list:
+            if "=" in tag:
+                key, value = tag.split("=", 1)
+                td[key] = value
+            else:
+                td[tag] = ""
+    return td
+
